# gda2html.py 
#
# Ralph Haugerud, U.S. Geological Survey, Seattle WA
#   rhaugerud@usgs.gov
#   20 February 2006
# 
# A Python script to turn a GDA archive directory into a narrative similar to
# to paper field notes. The narrative is written in HTML and can (1) be easily 
# edited using an HTML editor such as (free and WYSIWYG!) Mozilla Composer 
# (google for it) and (2) printed for a paper record.

# INSTALLATION
#
# You must have Python installed on your computer. Most ArcGIS users will have
# Python as part of their Arc installation. Otherwise, get Python from the web. 
# It is free and available for many operating systems. 
#
# Module dbf is not part of the standard Python installation. Obtain
# package dbfpy at http://www.fiby.at/dbfpy/.  Unpack it (it's a .tgz file)
# and place files dbf.py, binnutil.py, and strutil.py in--for Python 2.1 on 
# WinXP as installed by Arc 9--c:\python21\Lib\. 
#
# Module Image is part of the Python Imaging Library, available at
# http://www.pythonware.com/products/pil/. Download the version appropriate 
# for your system and install it. 
#
# The gda2html.py script should be in your working directory, perhaps
# the directory that contains your GDA archive subdirectories.

# USE
#
# Open an MSDOS command window. CD to the working directory. At the prompt, type
# 
# C:\FieldData> gda2html.py <archive_directory>
# 
# and <ENTER>, where <archive_directory> is the name of a GDA archive directory. 
# This script assumes that all image and text files (notes, photos) relevant to 
# an archive are contained within the archive directory. The HTML narrative is 
# created in the working directory, and will be named <archive_directory>.html.
#
# Gda2html.py makes no changes to the contents of the archive directory
# EXCEPT that it (1) renames all .JPGs to lower case and (2) creates thumbnails
# of all .jpgs which are named <imagefile>_sm.jpg
#
# My experience is that it is a pain to synchronize photoIDs between my digital 
# camera and GDA. So I expect to edit the narrative after the fact, to correlate 
# stations, photos, and photo notes. This is where Mozilla Composer comes into 
# play. Gda2html.py appends thumbnails, basic file information, and 
# links to the full-size image file to the end of the narrative, if the image 
# files are in the archive directory. Open the narrative file with Composer, 
# move the thumbnails and info arround, and add further comments as you see fit. 
# Think of this as the end-of-the-day inking of your field sheet. 
# 
# If a cheap inkjet printer is available, print the edited narrative for a paper
# record.

#
# Things to be fixed:
#  * 	Modify so works without photo or sample dbf files--or, even better, scans
#    	for any dbf with a STAID field (except for Stations.dbf, which should be 
#    	mandatory)
#  * 	Report station X and Y. Will require that these be saved to Stations.dbf
#  * 	Check for existence of .html file; so that will not overwrite existing file
#  * 	Try to match photos to proper stations
#  * 	Check: does script ignore deleted (but unpacked) records?
#
# and way down the road
#  *	sketch files?
#  *	overlays for photos?
#

import sys, glob, os, string, time, dbf, Image
from stat import *

programID = 'gda2html.py--version of 20 Feb 2006'

months = {'01':'Jan','02':'Feb','03':'Mar','04':'Apr',
          '05':'May','06':'June','07':'July','08':'Aug',
          '09':'Sept','10':'Oct','11':'Nov','12':'Dec'}

StaDbf = dbf.Dbf()
StrDbf = dbf.Dbf()
SampDbf = dbf.Dbf()
PhotDbf = dbf.Dbf()

def listRecord(aDbf,recNum):
	for fldName in aDbf.fieldNames():
		# add line breaks 
		if fldName in ('HOWLOC','FRED'): html.write('<br>\n')
		fldValue = aDbf[recNum][fldName]
		if fldName in ('AZI','INC'): fldValue = int(round(fldValue))
		if fldName not in ('STAID') and str(aDbf[recNum][fldName]) <> '':
			html.write('<i>'+fldName+':</i> '+str(fldValue))
			html.write(' &nbsp;&nbsp;')
	html.write('<br>\n')

def dateTime(StationID):
	if len(StationID)>= 12:
		Date = StationID[6:8]+' '+months[StationID[4:6]]+' 20'+StationID[2:4]
		ampm = 'am'
		hour = int(StationID[8:10])
		if hour >= 12: ampm = 'pm'
		if hour > 12: hour = hour - 12
		Time = str(hour)+':'+StationID[10:12]+' '+ampm
		return Date+' &nbsp;'+Time
	else: 
		return '  '

def openDbFiles():
	StaDbf.openFile(sourceDir+'/Stations.dbf', readOnly=1)
	StrDbf.openFile(sourceDir+'/Structure.dbf', readOnly=1)
	SampDbf.openFile(sourceDir+'/Samples.dbf', readOnly=1)
	PhotDbf.openFile(sourceDir+'/Photos.dbf', readOnly=1)

def makeThumbnails():
	# Windoze is case-insensitive. JPG and JPEG included for other OS
	for ext in ('.jpg','.jpeg','.JPG','.JPEG'):
		photos = glob.glob(sourceDir+'/*'+ext)
		for photo in photos:
			nphoto = string.lower(photo)
			os.rename(photo,nphoto)
			fn, ext = os.path.splitext(nphoto)
			im = Image.open(nphoto)
			if fn[-3:] <> '_sm':
				im.thumbnail((200,200),Image.ANTIALIAS)
				im.save(fn+'_sm.jpg',"JPEG")
				
def writeImages():
	img_sms = glob.glob(sourceDir+'/*_sm.JPG')
	for img_sm in img_sms:
		img = img_sm[:-7]+'.jpg'
		imgInfo = os.stat(img)
		html.write('<table border="1"><tr>\n')
		html.write('  <td><a href="')
		html.write(img+'" <img alt="thumbnail" src="'+img_sm+'"></a></td>\n')
		html.write('  <td>'+img+'<br>')
		html.write(str(imgInfo[ST_SIZE])+' bytes<br>')
		imgTime = time.ctime(imgInfo[ST_MTIME])
		html.write(imgTime+'<br></td>\n')
		html.write('</tr></table><br>\n')

def writeNoteFile(noteFileName):
	txtFile = open(noteFileName)
	txtFileLines = txtFile.readlines()
	html.write('<blockquote>')
	for aline in txtFileLines:
		html.write(aline[0])
		for n in range(1,len(aline)-2):
			if aline[n-1:n] == '  ':
				html.write('&nbsp;')
			else:
				html.write(aline[n])
		html.write('<br>\n')
	html.write('</blockquote>')
	txtFile.close()

def writeNarrative():
	html.write('<html>\n')
	html.write('<div style="text-align: right;">\n')
	html.write(time.ctime()+'<br></div>\n')
	for i1 in range(len(StaDbf)):
		StaRec = StaDbf[i1]
		StationID = StaRec['STAID']
		html.write('<a name="'+StationID+'"><b>'+StationID+'</b></a>')
		html.write('<small>')
		html.write(' &nbsp;&nbsp;&nbsp;'+dateTime(StationID)+'<br>\n')
		listRecord(StaDbf,i1)
		for i2 in range(len(StrDbf)):
			if StrDbf[i2]['STAID'] == StationID:
				listRecord(StrDbf, i2)
		for i2 in range(len(SampDbf)):
			if SampDbf[i2]['STAID'] == StationID:
				listRecord(SampDbf, i2)
		for i2 in range(len(PhotDbf)):
			if PhotDbf[i2]['STAID'] == StationID:
				listRecord(PhotDbf,i2)
		noteFileName = sourceDir+'/'+StationID+'.txt'
		if len(glob.glob(noteFileName)) > 0:
			writeNoteFile(noteFileName)
		html.write('</small><br>\n')
		html.write('<hr style="width: 100%; height: 2px;"><br>\n')
	writeImages()
	html.write('<br><br><small><i>'+programID+'</i></small>\n')
	html.write('</html>\n')

## MAIN PROGRAM ##
try:
	sourceDir = sys.argv[1]
	openDbFiles()
except IndexError:
	print '  IndexError: Did you specify an archive directory?'
	print
	print 'Usage: gda2html.py <dir>' 
	print '   where <dir> is a GDA archive directory'
	print
except IOError:
	print '  IOError: cannot find files Stations.dbf, Structure.dbf,' 
	print '     Photos.dbf, Samples.dbf in directory '+sourceDir
	print
else:
	try:
		makeThumbnails()
	except IOError:
		print 'IOError, unable to make thumbnails in '+sourceDir
		print
	else:
		try:
			html = open(sourceDir+'.html','w')
			writeNarrative()
			html.close()
		except IOError:
			print 'IOError, appears to be a problem writing '+sourceDir+'.html'
			print
